//
//  HumiditySensorController.m
//  HumiditySensor
//
//  Created by Phidgets on 2015-07-15.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "HumiditySensorController.h"

@implementation HumiditySensorController

- (void)windowWillClose:(NSNotification*)aNotification{
    if(ch){
        //Safely close the example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetHumiditySensor_delete(&ch);
        ch = NULL;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetHumiditySensor_create(&ch);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to create humidity sensor channel"];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        const char* resultString;
        Phidget_getErrorDescription(result, &resultString);
        NSString *errorString = [NSString stringWithFormat:@"Failed to initialize channel: %@",  [NSString stringWithUTF8String:resultString]];
        [self addError:errorString];
    }
    
    result = PhidgetHumiditySensor_setOnHumidityChangeHandler(ch, gotHumidityChangeData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to assign humidity change handler"];
    }
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to assign attach handler"];
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to assign detach handler"];
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to assign error handler"];
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}

#pragma mark Event callbacks
static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}
void gotHumidityChangeData(PhidgetHumiditySensorHandle phid, void *context, double percentHumidity){
        [(__bridge id)context performSelectorOnMainThread:@selector(onHumidityChangeHandler:)
                                               withObject:[NSNumber numberWithDouble:percentHumidity]
                                            waitUntilDone:NO];
}

#pragma mark Attach, detach, data, and error events
- (void)onAttachHandler{
    uint32 minDataInterval, maxDataInterval, dataInterval;
    double minHumidityChangeTrigger,maxHumidityChangeTrigger, humidityChangeTrigger;
    
    //Get information from channel which will allow us to configure the GUI properly
    PhidgetHumiditySensor_getMaxDataInterval(ch, &maxDataInterval);
    PhidgetHumiditySensor_getMinDataInterval(ch, &minDataInterval);
    PhidgetHumiditySensor_getDataInterval(ch, &dataInterval);
    PhidgetHumiditySensor_getMaxHumidityChangeTrigger(ch, &maxHumidityChangeTrigger);
    PhidgetHumiditySensor_getMinHumidityChangeTrigger(ch, &minHumidityChangeTrigger);
    PhidgetHumiditySensor_getHumidityChangeTrigger(ch, &humidityChangeTrigger);
    
    //Adjust GUI based on information from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [dataBox setHidden:NO];
    [settingsBox setHidden:NO];
    
    //Configure the data interval slider with information read from channel
    [maxDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",maxDataInterval]];
    [minDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",minDataInterval]];
    [dataIntervalSlider setMaxValue:(double)maxDataInterval];
    [dataIntervalSlider setMinValue:(double)minDataInterval];
    [dataIntervalSlider setIntValue:dataInterval];
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", dataInterval]];
    
    //Configure the change trigger slider with information read from channel
    [maxChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%0.1f%%",maxHumidityChangeTrigger]];
    [minChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%0.1f%%",minHumidityChangeTrigger]];
    [changeTriggerSlider setMaxValue:maxHumidityChangeTrigger];
    [changeTriggerSlider setMinValue:minHumidityChangeTrigger];
    [changeTriggerSlider setDoubleValue:humidityChangeTrigger];
    [changeTriggerText setStringValue:[NSString stringWithFormat:@"%0.1f%%", humidityChangeTrigger]];
    
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow maxSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onDetachHandler{
    //Taking items out of view
    [settingsBox setHidden:YES];
    [dataBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onHumidityChangeHandler:(NSNumber *)percentHumidity{
    [humidityText setStringValue:[NSString stringWithFormat:@"%0.2f%%", percentHumidity.doubleValue]];
}

static int errorCounter = 0;
-(void)errorHandler:(NSArray *)errorEventData{
    [self addError:[errorEventData objectAtIndex:1]];
}

-(void)addError:(NSString *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",errorString]];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}

#pragma mark GUI controls
- (IBAction)setChangeTrigger:(id)sender{
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if([currentEvent type] == NSEventTypeLeftMouseUp){
        result = PhidgetHumiditySensor_setHumidityChangeTrigger(ch, [changeTriggerSlider doubleValue]);
        if(result != EPHIDGET_OK){
            const char* errorString;
            Phidget_getErrorDescription(result, &errorString);
            [self addError:[NSString stringWithFormat:@"Failed setting change trigger:\n%@",[NSString stringWithUTF8String:errorString]]];
            return;
        }
    }
    [changeTriggerText setStringValue:[NSString stringWithFormat:@"%0.1f%%", [changeTriggerSlider doubleValue]]];
}

- (IBAction)setDataInterval:(id)sender{
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if([currentEvent type] == NSEventTypeLeftMouseUp){
        result = PhidgetHumiditySensor_setDataInterval(ch, [dataIntervalSlider intValue]);
        if(result != EPHIDGET_OK){
            const char* errorString;
            Phidget_getErrorDescription(result, &errorString);
            [self addError:[NSString stringWithFormat:@"Failed setting data interval:\n%@",[NSString stringWithUTF8String:errorString]]];
            return;
        }
    }
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", [dataIntervalSlider intValue]]];
}

@end
